package com.gic.haoban.manage.service.service.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.dto.KeyDataDTO;
import com.gic.haoban.manage.api.enums.KeyDataEnum;
import com.gic.haoban.manage.service.dao.mapper.TabKeyDataMapper;
import com.gic.haoban.manage.service.entity.TabKeyData;
import com.gic.haoban.manage.service.service.KeyDataService;

@Service("keyDataService")
public class KeyDataServiceImpl implements KeyDataService {

    public static String[] PADDING = new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C",
            "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X",
            "Y", "Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
            "t", "u", "v", "w", "x", "y", "z" };

    @Autowired
    private TabKeyDataMapper tabKeyDataMapper;

    @Override
    public KeyDataDTO saveData(String enterpriseId , String relationId , String data, KeyDataEnum dataEnum) {
        KeyDataDTO dto = new KeyDataDTO();
        dto.setRelationId(relationId);
        dto.setEnterpriseId(enterpriseId);
        dto.setDataKey(generateKey(dataEnum));
        dto.setCreateTime(new Date());
        dto.setId(UniqueIdUtils.uniqueLong());
        dto.setData(data);
        dto.setType(dataEnum.getType());
        TabKeyData entity = EntityUtil.changeEntityByJSON(TabKeyData.class, dto);
        this.tabKeyDataMapper.insert(entity);
        return dto;
    }
    
    @Override
    public String saveHmData(String enterpriseId , String unionid, Long linkId, Long hmId) {
        JSONObject json = new JSONObject();
        json.put("unid", unionid);
        json.put("ljid", linkId);
        json.put("hmid", hmId);
        KeyDataDTO data = this.saveData(enterpriseId,String.valueOf(linkId), json.toString(), KeyDataEnum.DYNAMIC_HM);
        return data.getDataKey();
    }

    @Override
    public KeyDataDTO getDataByKey(String key, KeyDataEnum dataEnum , String relationId) {
        TabKeyData tab = this.tabKeyDataMapper.selectByKey(key, dataEnum.getType() ,relationId);
        if (null != tab) {
            return EntityUtil.changeEntityByJSON(KeyDataDTO.class, tab);
        }
        return null;
    }
    
    @Override
    public String getMobile(String key, String relationId) {
        KeyDataDTO data = this.getDataByKey(key, KeyDataEnum.MOBILE, relationId) ;
        if(null != data) {
            return JSON.parseObject(data.getData()).getString("ph") ;
        }
        return null;
    }

    public static String generateKey(KeyDataEnum dataEnum) {
        if (dataEnum.equals(KeyDataEnum.MOBILE)) {
            StringBuilder sb = new StringBuilder();
            String uuid = ToolUtil.randomUUID();
            int length = 6;
            int interval = (int) (uuid.length() / length);
            String str;
            for (int i = 0; i < length; i++) {
                if (i == length - 1) {
                    str = uuid.substring(length * interval);
                } else {
                    str = uuid.substring(i * interval, i * interval + interval);
                }
                int x = Integer.parseInt(str, 16);
                sb.append(PADDING[x % 62]);
            }
            sb.append(PADDING[(int) (Math.random() * 62)]);
            return sb.toString();
        }
        return UniqueIdUtils.uniqueLongHex();
    }

    public static void main(String[] args) {
        Set<String> set = new HashSet<>();
        for (int i = 0; i < 1000000; i++) {
            String s = generateKey(KeyDataEnum.MOBILE);
            set.add(s);
        }
        System.out.println(set.size());
    }

}
